#!/usr/local/bin/perl

#===========================================================
# WwwBoard Ver 3.0
# Copyright (C) 1997-2021 とほほ
# https://www.tohoho-web.com/
# フリーソフト・再配布/改造/流用可能・確認メール不要
#===========================================================

# 1997/03/23 初版
# 2000/03/20 Ver2.61 シンタックスチェック
# 2001/04/30 Ver2.62 設置説明文の変更
# 2021/05/04 Ver3.0 全面改訂

#
# メッセージの最大数
#   この個数を超えたメッセージは古いものから削除されます
#   最大数を制限しない場合は 0 を指定してください
#
$max_msgs = 100;

#
# Return URL
#
$return_url = "../index.html";

#
# タイムゾーン
#
$ENV{'TZ'} = "JST-9";

#
# 曜日定数
#
@wdays = ( "日", "月", "火", "水", "木", "金", "土", "日" );

#
# HTMLサニタイジング
#
sub html {
	local($msg) = $_[0];
	$msg =~ s/&/&amp;/g;
	$msg =~ s/</&lt;/g;
	$msg =~ s/>/&gt;/g;
	$msg =~ s/"/&quot;/g;
	$msg =~ s/'/&#39;/g;
	$msg =~ s/ /&nbsp;/g;
	return $msg;
}

#
# テストモード
#
if ($ARGV[0] eq "test") {
	print "Content-type: text/html\n";
	print "\n";
	print "<!doctype html>\n";
	print "<html>\n";
	print "<head>\n";
	print "<meta charset='utf-8'>\n";
	print "<meta name='viewport' content='width=device-width,initial-scale=1.0'>\n";
	print "<title>wwwboard test</title>\n";
	print "</head>\n";
	print "<body>\n";
	print "<p>CGIスクリプトは正常に動作しています。</p>\n";
	if (! -d "lock") {
		print "<p>NG. lockディレクトリが存在しません。</p>\n";
	} elsif (! -w "lock") {
		print "<p>NG. lockディレクトリに書き込みができません。</p>\n";
	} elsif (-d "lock/wwwboard.loc") {
		print "<p>NG. lock/wwwboard.loc が残っています。</p>\n";
	}
	if (! -f "wwwboard.dat") {
		print "<p>NG. wwwboard.dat が存在しません。</p>\n";
	} elsif (! -w "wwwboard.dat") {
		print "<p>NG. wwwboard.dat に書き込みできません。</p>\n";
	}
	if (! -f "wwwboard.tmp") {
		print "<p>NG. wwwboard.tmp が存在しません。</p>\n";
	} elsif (! -w "wwwboard.tmp") {
		print "<p>NG. wwwboard.tmp に書き込みできません。</p>\n";
	}
	print "</body>\n";
	print "</html>\n";
	exit(0);
}

#
# フォームに入力された値を読み込む
#
if ($ENV{'REQUEST_METHOD'} eq "POST") {
	read(STDIN, $query_string, $ENV{'CONTENT_LENGTH'});
	@a = split(/&/, $query_string);
	foreach $x (@a) {
		($name, $value) = split(/=/, $x);
		$value =~ tr/+/ /;
		$value =~ s/%([0-9a-fA-F][0-9a-fA-F])/pack("C", hex($1))/eg;
		$FORM{$name} = $value;
	}
}

#
# ロックをかける
#
foreach $i ( 1, 2, 3, 4, 5, 6 ) {
	if (mkdir("lock/wwwboard.loc", 0755)) {
		last;
	} elsif ($i == 1) {
		($mtime) = (stat("lock/wwwboard.loc"))[9];
		if ($mtime < time() - 600) {
			rmdir("lock/wwwboard.loc");
		}
	} elsif ($i < 6) {
		sleep(2);
	} else {
		print "Content-type: text/html\n";
		print "\n";
		print "<!doctype html>\n";
		print "<html>\n";
		print "<head>\n";
		print "<meta charset='utf-8'>\n";
		print "<meta name='viewport' content='width=device-width,initial-scale=1.0'>\n";
		print "<title>掲示板</title>\n";
		print "</head>\n";
		print "<body>\n";
		print "<h1>掲示板</h1>\n";
		print "<hr>\n";
		print "只今、掲示板が混雑しております。しばらくお待ちの上、";
		print "再度アクセスお願いします。\n";
		print "<hr>\n";
		print "</body>\n";
		print "</html>\n";
		exit(1);
	}
}

#
# シグナル停止した際にロックを解除するようにする
#
sub sigexit { rmdir("lock/wwwboard.loc"); exit(0); }
$SIG{'PIPE'} = $SIG{'INT'} = $SIG{'HUP'} = $SIG{'QUIT'} = $SIG{'TERM'} = "sigexit";

#
# データファイルに書き込む
#
if ($FORM{'message'} ne "") {
	#
	# 日時を得る
	#
	($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime(time);
	$date = sprintf("%04d/%02d/%02d(%s) %02d:%02d:%02d",
		$year + 1900, $mon + 1, $mday, $wdays[$wday],
		$hour, $min, $sec);

	#
	# テンポラリファイル(.tmp)にメッセージを書き込む
	#
	$message = $FORM{'message'};
	$message =~ s/\r\n/\n /g;
	open(IN, "wwwboard.dat");
	open(OUT, "> wwwboard.tmp");
	print OUT "name: $FORM{'name'}\n";
	print OUT "date: $date\n";
	print OUT "message: $message\n";
	while (<IN>) {
		print OUT;
	}
	close(IN);
	close(OUT);

	#
	# テンポラリファイル(.tmp)をデータファイル(.dat)にコピーする
	#
	open(IN, "wwwboard.tmp");
	open(OUT, "> wwwboard.dat");
	$msgs = 0;
	while (<IN>) {
		if ($_ =~ /^name:/) {
			if ($max_msgs != 0) {
				if ($msgs++ >= $max_msgs) {
					last;
				}
			}
		}
		print OUT;
	}
	close(IN);
	close(OUT);
}

#
# 戻るリンク
#
if ($return_url ne "") {
    $return_html = "<div class='return-url'><a href='$return_url'>[戻る]</a></div>\n";
}

#
# HTMLを書き出す(前半)
#
print <<"EOF";
Content-Type: text/html

<!doctype html>
<html>
<head>
<meta charset="utf-8">
<meta name="viewport" content="width=device-width,initial-scale=1.0">
<title>掲示板</title>
<link rel="stylesheet" href="style.css">
</head>
<body>
<h1>掲示板</h1>
$return_html
<form method="POST" action="wwwboard.cgi">
 <div class="name-box">
  <input type="text" name="name" value="$FORM{'name'}" placeholder="名前">
 </div>
 <div class="message-box">
  <textarea name="message" placeholder="メッセージ"></textarea>
 </div>
 <button>送信 / 再読み込み</button>
</form>
EOF

#
# メッセージを書き出す
#
$mflag = 0;
open(IN, "wwwboard.dat");
while (<IN>) {
	if ($_ =~ /^name: (.*)\n/) {
		if ($mflag == 1) {
			print " </div>\n";
			print "</div>\n";
			$mflag = 0;
		}
		print "<div class='block'>\n";
		print " <div class='name'>" . html($1) . "</div>\n";
	} elsif ($_ =~ /^date: (.*)\n/) {
		print " <div class='date'>" . html($1) . "</div>\n";
	} elsif ($_ =~ /^message: (.*)\n/) {
		$mflag = 1;
		print " <div class='message'>\n";
		print html($1) . "<br>\n";
	} elsif ($_ =~ /^ (.*)\n/) {
		print html($1) . "<br>\n";
	}
}
if ($mflag) {
	print " </div>\n";
	print "</div>\n";
}
close(IN);

#
# HTMLを書き出す(後半)
#
print <<"EOF";
</body>
</html>
EOF

#
# ロックを解除する
#
rmdir("lock/wwwboard.loc");
